/* ikna.c */
#define MAX_CARD_ID_BUFFER_SIZE 16
#define MAX_LOCK_FILE_BUFFER_SIZE 100
#define MAX_TIME_T_BUFFER_SIZE 16
#define MAX_DAILY_REVIEWS_BUFFER_SIZE 11
#define MAX_SETTINGS_FILE_LINE_BUFFER_SIZE 30
#define MAX_LOG_FILE_LINE_BUFFER_SIZE 90

#define SETTINGS_FILE_MAX_FIELDS 2
#define LOG_FILE_MAX_FIELDS 6
#define DATE_MAX_FIELDS 3

typedef unsigned int uint;
typedef long unsigned int ulong;

enum {
	OPTION_NOTHING,
	OPTION_FRONT,
	OPTION_BACK,
	OPTION_CORRECT,
	OPTION_INCORRECT,
	OPTION_STATUS,
	OPTION_NEW,
	OPTION_LIST,
	OPTION_DELETE,
	OPTION_UNDELETE,
	OPTION_IMPORT,
	OPTION_EXPORT,
	OPTION_INIT
};

enum {
	LOCK_FIELD_CURRENT_CARD = 0,
	LOCK_FIELD_LAST_REVIEWED = 1,
	LOCK_FIELD_REVIEW_COUNT = 2,
	LOCK_FIELD_NEW_CARDS_COUNT = 3,
	LOCK_FIELD_MAX = 5 /* Must be 2 + previous enum size */
};

struct card_data {
	int ready_for_review;
	uint id;
	uint interval;
	time_t review_time;
	float multiplier;
	char card_action;
};


struct settings {
	uint max_daily_reviews;
	uint max_daily_new_cards;
	uint default_interval;
	float default_multiplier;
};

struct lock_fields {
	time_t last_reviewed;
	uint current_card_id;
	int has_current_card;
	uint review_count;
	uint new_cards_count;
};

static void print_card(const char *, uint);
static uint read_file_to_char(FILE *, int *, char *, uint, char);
static uint split_string_on_char(char *, char **, char, uint);
static uint parse_card_data(const char *, struct card_data **);
static void append_log_file(const char *, uint, char, time_t, uint, float);
static struct lock_fields get_lock_fields(char *);
static void set_lock_fields(char *, struct lock_fields);
static uint current_card_id(int *);
static void set_next_current_card(struct card_data *, uint);
static time_t epoch_today_day_start(void);
static void reset_review_count_on_new_day(void);
static int reviewing_cards_today(struct card_data *, uint);
static uint validate_card_id_argument(const char *, uint);
static int is_string_valid_int(const char *string);
static struct settings parse_settings_file(const char *);
static void die(const char *, ...);
static FILE *xfopen(const char *, const char *);
static void *xmalloc(size_t);
static void *xrealloc(void *, size_t);
static void get_card_faces(const char *, char **, char **);
static void new_card(const char *, const char *, uint);
static ulong string_to_ulong(const char *);
/* static long string_to_long(const char *); */

/* csv.c */
#define PARSE_BUFF_SIZE_DEFAULT 128
#define FIELDS_BUFF_SIZE_DEFAULT 64

struct csv_field {
	uint type;
	char *content;
};

enum {
	IN_FRONT,
	IN_BACK
};

enum {
	FIELD_TYPE_NORMAL,
	FIELD_TYPE_QUOTED
};

static void csv_export(uint, char);
static void csv_import(uint, char);
static void print_string_duplicate_double_quotes(char *);
static void delete_index_from_string(char *, uint);
static void fix_quoted_field_string(char *);
